@@ -1,92 +1,66 @@
-#include "../../uwsgi.h"
+#include <uwsgi.h>
 
 #ifdef UWSGI_ROUTING
 
 extern struct uwsgi_server uwsgi;
 
-int uwsgi_routing_func_http(struct wsgi_request *wsgi_req, struct uwsgi_route *ur) {
+static int uwsgi_routing_func_http(struct wsgi_request *wsgi_req, struct uwsgi_route *ur) {
 
 	// mark a route request
-        wsgi_req->status = -17;
+        wsgi_req->via = UWSGI_VIA_ROUTE;
 
 	// get the http address from the route
 	char *addr = ur->data;
 
-	char *uri = NULL;
-	uint16_t uri_len = 0;
+	struct uwsgi_buffer *ub_url = NULL;
 	if (ur->data3_len) {
-		uri = uwsgi_regexp_apply_ovec(wsgi_req->uri, wsgi_req->uri_len, ur->data3, ur->data3_len, ur->ovector, ur->ovn);
-		uri_len = strlen(uri);
+		char **subject = (char **) (((char *)(wsgi_req))+ur->subject);
+        	uint16_t *subject_len = (uint16_t *)  (((char *)(wsgi_req))+ur->subject_len);
+		ub_url = uwsgi_routing_translate(wsgi_req, ur, *subject, *subject_len, ur->data3, ur->data3_len);
+        	if (!ub_url) return UWSGI_ROUTE_BREAK;
 	}
 
 
 	// convert the wsgi_request to an http proxy request
-	struct uwsgi_buffer *ub = uwsgi_to_http(wsgi_req, ur->data2, ur->data2_len, uri, uri_len);	
+	struct uwsgi_buffer *ub = uwsgi_to_http(wsgi_req, ur->data2, ur->data2_len, ub_url ? ub_url->buf : NULL, ub_url ? ub_url->pos : 0);	
 	if (!ub) {
-		if (uri) free(uri);
+		if (ub_url) uwsgi_buffer_destroy(ub_url);
 		uwsgi_log("unable to generate http request for %s\n", addr);
                 return UWSGI_ROUTE_NEXT;
 	}
 
-	if (uri) free(uri);
+	if (ub_url) uwsgi_buffer_destroy(ub_url);
+
+	// amount of body to send
+	size_t remains = wsgi_req->post_cl - wsgi_req->proto_parser_remains;
+	// append remaining body...
+	if (wsgi_req->proto_parser_remains > 0) {
+		if (uwsgi_buffer_append(ub, wsgi_req->proto_parser_remains_buf, wsgi_req->proto_parser_remains)) {
+			uwsgi_log("unable to generate http request for %s\n", addr);
+               		return UWSGI_ROUTE_NEXT;
+		}
+		wsgi_req->proto_parser_remains = 0;
+	}
 
 	// ok now if have offload threads, directly use them
 	if (wsgi_req->socket->can_offload) {
         	if (!uwsgi_offload_request_net_do(wsgi_req, addr, ub)) {
-                	wsgi_req->status = -30;
+                	wsgi_req->via = UWSGI_VIA_OFFLOAD;
 			return UWSGI_ROUTE_BREAK;
                 }
 	}
 
-	// connect to the http server
-	int http_fd = uwsgi_connect(addr, uwsgi.shared->options[UWSGI_OPTION_SOCKET_TIMEOUT], 0);
-	if (http_fd < 0) {
-		uwsgi_log("unable to connect to host %s\n", addr);
-		return UWSGI_ROUTE_NEXT;
-	}
-
-	// send the request
-	if (uwsgi_buffer_send(ub, http_fd)) {
-		uwsgi_log("error routing request to http server %s\n", addr);
-		close(http_fd);
-		uwsgi_buffer_destroy(ub);
-                return UWSGI_ROUTE_NEXT;
-	}
-
-	ssize_t ret;
-
-	// pipe the body
-	if (wsgi_req->post_cl > 0) {
-		int post_fd = wsgi_req->poll.fd;
-		if (wsgi_req->async_post) {
-			post_fd = fileno((FILE *)wsgi_req->async_post);
-		}
-		ret = uwsgi_pipe_sized(post_fd, http_fd, wsgi_req->post_cl, 0);
-		if (ret < 0) {
-			uwsgi_log("error routing request body (%llu bytes) to http server %s\n", (unsigned long long) wsgi_req->post_cl, addr);
-			close(http_fd);
-                	uwsgi_buffer_destroy(ub);
-			return UWSGI_ROUTE_BREAK;
-		}
-	}
-
-	// pipe the response
-	ret = uwsgi_pipe(http_fd, wsgi_req->poll.fd, 0);
-	if (ret > 0) {
-		wsgi_req->response_size += ret;
-	}
-	else {
+	if (uwsgi_proxy_nb(wsgi_req, addr, ub, remains, uwsgi.shared->options[UWSGI_OPTION_SOCKET_TIMEOUT])) {
 		uwsgi_log("error routing request to http server %s\n", addr);
 	}
 
-	close(http_fd);
 	uwsgi_buffer_destroy(ub);
 
 	return UWSGI_ROUTE_BREAK;
 
 }
 
-int uwsgi_router_http(struct uwsgi_route *ur, char *args) {
+static int uwsgi_router_http(struct uwsgi_route *ur, char *args) {
 
 	ur->func = uwsgi_routing_func_http;
 	ur->data = (void *) args;
@@ -109,7 +83,7 @@ int uwsgi_router_http(struct uwsgi_route *ur, char *args) {
 }
 
 
-void router_http_register(void) {
+static void router_http_register(void) {
 
 	uwsgi_register_router("http", uwsgi_router_http);
 }
