@@ -1,6 +1,6 @@
 # uWSGI build system
 
-uwsgi_version = '1.9'
+uwsgi_version = '1.9.1'
 
 import os
 import re
@@ -64,7 +64,6 @@ report['pcre'] = False
 report['matheval'] = False
 report['routing'] = False
 report['capabilities'] = False
-report['ini'] = False
 report['yaml'] = False
 report['json'] = False
 report['ldap'] = False
@@ -422,6 +421,20 @@ def build_uwsgi(uc, print_only=False):
     for pb in post_build:
         pb(uc)
 
+def open_profile(filename):
+    if filename.startswith('http://') or filename.startswith('https://') or filename.startswith('ftp://'):
+        wrapped = False
+        try:
+            import urllib2
+        except:
+            import urllib.request
+            wrapped = True
+
+        if wrapped:
+            import io
+            return io.TextIOWrapper(urllib.request.urlopen(filename), encoding='utf-8')
+        return urllib2.urlopen(filename)
+    return open(filename)
 
 class uConf(object):
 
@@ -429,8 +442,6 @@ class uConf(object):
         self.config = ConfigParser.ConfigParser()
         if not mute:
             print("using profile: %s" % filename)
-        if not os.path.exists(filename):
-            raise Exception("profile not found !!!")
 
         if os.path.exists('uwsgibuild.lastprofile'):
             ulp = open('uwsgibuild.lastprofile','r')
@@ -443,13 +454,13 @@ class uConf(object):
         ulp.write(filename)
         ulp.close()
 
-        self.config.read(filename)
+        self.config.readfp(open_profile(filename))
         self.gcc_list = ['core/utils', 'core/protocol', 'core/socket', 'core/logging', 'core/master', 'core/master_utils', 'core/emperor',
             'core/notify', 'core/mule', 'core/subscription', 'core/stats', 'core/sendfile', 'core/async', 'core/master_checks',
             'core/offload', 'core/io', 'core/static', 'core/websockets', 'core/spooler', 'core/snmp', 'core/exceptions',
             'core/setup_utils', 'core/clock', 'core/init', 'core/buffer', 'core/reader', 'core/writer', 'core/alarm',
             'core/plugins', 'core/lock', 'core/cache', 'core/daemons', 'core/errors', 'core/hash', 'core/master_events',
-            'core/queue', 'core/event', 'core/signal', 'core/strings', 'core/progress', 'core/timebomb',
+            'core/queue', 'core/event', 'core/signal', 'core/strings', 'core/progress', 'core/timebomb', 'core/ini',
             'core/rpc', 'core/gateway', 'core/loop', 'core/rb_timers', 'core/uwsgi']
         # add protocols
         self.gcc_list.append('proto/base')
@@ -532,7 +543,8 @@ class uConf(object):
             for option in self.config.options('uwsgi'):
                 interpolations[option] = self.get(option)
             iconfig = ConfigParser.ConfigParser(interpolations)
-            iconfig.read(inherit)
+            iconfig.readfp(open_profile(inherit))
+
             for opt in iconfig.options('uwsgi'):
                 if not self.config.has_option('uwsgi', opt):
                     self.set(opt, iconfig.get('uwsgi', opt))
@@ -615,7 +627,6 @@ class uConf(object):
             self.libs.append('-lkvm')
 
         if uwsgi_os == 'Haiku':
-            self.set('async', 'false')
             self.libs.remove('-rdynamic')
             self.libs.remove('-lpthread')
             self.libs.append('-lroot')
@@ -923,11 +934,6 @@ class uConf(object):
         self.cflags.append('-DUWSGI_VERSION_REVISION="' + uver_rev + '"')
         self.cflags.append('-DUWSGI_VERSION_CUSTOM="\\"' + uver_custom + '\\""')
 
-        if self.get('ini'):
-            self.cflags.append("-DUWSGI_INI")
-            self.gcc_list.append('core/ini')
-            report['ini'] = True
-
         if self.get('yaml'):
             self.cflags.append("-DUWSGI_YAML")
             self.gcc_list.append('core/yaml')
@@ -1081,38 +1087,41 @@ def build_plugin(path, uc, cflags, ldflags, libs, name = None):
         print("Error: unable to find directory '%s'" % path)
         sys.exit(1)
 
-    sys.path.insert(0, path)
-    import uwsgiplugin as up
-    if sys.argv[1] != '--plugin':
-        reload(up)
+    up = {}
+    try:
+        execfile('%s/uwsgiplugin.py' % path, up)
+    except:
+        f = open('%s/uwsgiplugin.py' % path)
+        exec(f.read(), up)
+        f.close()
 
     requires = []
 
     p_cflags = cflags[:]
     p_ldflags = ldflags[:]
 
-    p_cflags += up.CFLAGS
-    p_ldflags += up.LDFLAGS
-    p_libs = up.LIBS
+    p_cflags += up['CFLAGS']
+    p_ldflags += up['LDFLAGS']
+    p_libs = up['LIBS']
 
     post_build = None
 
     try:
-        requires = up.REQUIRES
+        requires = up['REQUIRES']
     except:
         pass
 
     try:
-        post_build = up.post_build
+        post_build = up['post_build']
     except:
         pass
 
     p_cflags.insert(0, '-I.')
 
     if name is None:
-        name = up.NAME
+        name = up['NAME']
     else:
-        p_cflags.append("-D%s_plugin=%s_plugin" % (up.NAME, name))
+        p_cflags.append("-D%s_plugin=%s_plugin" % (up['NAME'], name))
 
     try:
         for opt in uc.config.options(name):
@@ -1132,7 +1141,7 @@ def build_plugin(path, uc, cflags, ldflags, libs, name = None):
     if uwsgi_os == 'Darwin':
         shared_flag = '-dynamiclib -undefined dynamic_lookup'
 
-    for cfile in up.GCC_LIST:
+    for cfile in up['GCC_LIST']:
         if cfile.endswith('.a'): 
             gcc_list.append(cfile)
         elif not cfile.endswith('.c') and not cfile.endswith('.cc') and not cfile.endswith('.m'):
@@ -1182,9 +1191,6 @@ def build_plugin(path, uc, cflags, ldflags, libs, name = None):
         except:
             pass
 
-    #for ofile in up.OBJ_LIST:
-    #    gcc_list.insert(0,ofile)
-
     need_pic = ' -fPIC'
     # on cygwin we do not need PIC
     if uwsgi_os.startswith('CYGWIN'):
